from CvPythonExtensions import *
import PyHelpers
import sdToolKit as sd
import math
gc = CyGlobalContext()





class Disasters:
    
    def __init__(self):
        ## Globals
        self.iBaseDisasterChance = 5 ## chance that a disaster will occur at all
        self.iDamageVariance = 10 ## percent of variance in damage effects
        self.iDamageBuildingThreshold = 25 ## must be this much damage to reset a building
        self.iMaxSearchRange = 10 ## maximum range for disaster to find target to move to
        
        
    def loadXML(self, xmlFile):
        if sd.sdEntityExists('Disasters', 'DisasterBonesList'):
            pDisasterBonesList = sd.sdGetVal('Disasters','DisasterBonesList', 'DisasterBoneses')
            for line in xmlFile.readlines():
                if "##" in line: continue
                if "<Disaster>" in line:
                    pDisaster = DisasterBones()
                elif "<DisasterTag>" in line:
                    pDisaster.sDisasterTag = self.CutString(line)
                elif "<iWeight>" in line:
                    pDisaster.iWeight = int(self.CutString(line))
                elif "<iMinLifeTime>" in line:
                    pDisaster.iMinLifeTime = int(self.CutString(line))
                elif "<iMaxLifeTime>" in line:
                    pDisaster.iMaxLifeTime = int(self.CutString(line))
                elif "<PrimaryEffect>" in line:
                    pDisaster.sPrimaryEffect = self.CutString(line)
                elif "<iPrimaryEffectTimer>" in line:
                    pDisaster.iPrimaryEffectTimer = int(self.CutString(line))
                elif "<SecondaryEffect>" in line:
                    pDisaster.sSecondaryEffect = self.CutString(line)
                elif "<iSecondaryEffectTimer>" in line:
                    pDisaster.iSecondaryEffectTimer = int(self.CutString(line))
                elif "<iMoveSpeed>" in line:
                    pDisaster.iMoveSpeed = int(self.CutString(line))
                elif "<Target>" in line:
                    pDisaster.sTarget = self.CutString(line)
                elif "<bTargetLand>" in line:
                    pDisaster.bTargetLand = bool(eval(self.CutString(line)))
                elif "<Damage>" in line:
                    pDamage = Damage()
                elif "<DamageDescription>" in line:
                    pDamage.sDamageDescription = self.CutString(line)
                elif "<iThisDamageChance>" in line:
                    pDamage.iChance = int(self.CutString(line))
                elif "<iMinBuildingDamage>" in line:
                    pDamage.iBuildingMinDamage = int(self.CutString(line))
                elif "<iMaxBuildingDamage>" in line:
                    pDamage.iBuildingMaxDamage = int(self.CutString(line))
                elif "<iMinImprovementDamage>" in line:
                    pDamage.iImprovementMinDamage = int(self.CutString(line))
                elif "<iMaxImprovementDamage>" in line:
                    pDamage.iImprovementMaxDamage = int(self.CutString(line))
                elif "<iMinUnitDamage>" in line:
                    pDamage.iUnitMinDamage = int(self.CutString(line))
                elif "<iMaxUnitDamage>" in line:
                    pDamage.iUnitMaxDamage = int(self.CutString(line))
                elif "<iMinPopulationDamage>" in line:
                    pDamage.iPopulationMinDamage = int(self.CutString(line))
                elif "<iMaxPopulationDamage>" in line:
                    pDamage.iPopulationMaxDamage = int(self.CutString(line))
                elif "<iDamageRadius>" in line:
                    pDamage.iDamageMaxDistance = int(self.CutString(line))
                elif "<iDamageCenterDistance>" in line:
                    pDamage.iDamageCenter = int(self.CutString(line))
                elif "<bInverseSquareDropoff>" in line:
                    pDamage.bInverseSquareDropoff = bool(eval((self.CutString(line))))
                elif "<iLinearDropoffPercent>" in line:
                    pDamage.iLinearDropoffPercent = int(self.CutString(line))
                elif "<DamageArtEffect>" in line:
                    pDamage.sEffect = self.CutString(line)
                elif "</Damage>" in line:
                    pDisaster.lDamages.append(pDamage)
                elif "<Spread>" in line:
                    pSpread = Spread()
                elif "<SpreadDisasterType>" in line:
                    pSpread.sSpreadDisasterType = self.CutString(line)
                elif "<iSpreadChance>" in line:
                    pSpread.iSpreadChance = int(self.CutString(line))
                elif "<bSpreadAdjacent>" in line:
                    pSpread.bSpreadAdjacent = bool(eval(self.CutString(line)))
                elif "<bSpreadOwnTile>" in line:
                    pSpread.bSpreadOwnTile = bool(eval(self.CutString(line)))
                elif "<bTradeRouteSpread>" in line:
                    pSpread.bTradeRouteSpread = bool(eval(self.CutString(line)))
                elif "<SpreadChanceIncrease>" in line:
                    pSpread.lSpreadChanceIncreasers.append(self.CutString(line))
                elif "<iIncrease>" in line:
                    pSpread.lSpreadChanceIncreases.append(int(self.CutString(line)))
                elif "</Spread>" in line:
                    pDisaster.lSpreads.append(pSpread)
                elif "</Disaster>" in line:
                    pDisasterBonesList.append(pDisaster)
            sd.sdSetVal('Disasters', 'DisasterBonesList', 'DisasterBoneses', pDisasterBonesList)
            
            
    
    def initializeDisasterList(self):
        sd.sdEntityInit('Disasters', 'DisasterList', {'Disasters': []})
        sd.sdEntityInit('Disasters','DisasterBonesList',{'DisasterBoneses': []})
        MyFile=open("Mods/Disasters/Assets/XML/DisasterInfos.xml")
        
        self.loadXML(MyFile)
        MyFile.close()

    def spawnDisasterByType(self, sDisasterType, x, y):
        lDisasterBoneses = sd.sdGetVal('Disasters','DisasterBonesList', 'DisasterBoneses')
        for pDisasterBones in lDisasterBoneses:
            if pDisasterBones.sDisasterTag == sDisasterType:
                self.spawnDisasterByBones(pDisasterBones, x, y)
                

    def spawnDisasterByBones(self, pDisasterBones, x, y):
        pDisaster = Disaster(pDisasterBones)
        pDisaster.X = x
        pDisaster.Y = y
        if sd.sdEntityExists('Disasters', 'DisasterList'):
            lDisasterList = sd.sdGetVal('Disasters', 'DisasterList', 'Disasters')
            lDisasterList.append(pDisaster)
            sd.sdSetVal('Disasters', 'DisasterList', 'Disasters', lDisasterList)
    
    def spawnDisasterByTypeList(self, sDisasterType, x, y, lDisasters):
        lDisasterBoneses = sd.sdGetVal('Disasters','DisasterBonesList', 'DisasterBoneses')
        for pDisasterBones in lDisasterBoneses:
            if pDisasterBones.sDisasterTag == sDisasterType:
                pNewDisaster = Disaster(pDisasterBones)
                pNewDisaster.X = x
                pNewDisaster.Y = y
                lDisasters.append(pNewDisaster)


    def doDisasterArt(self, iCounter):
        if sd.sdEntityExists('Disasters','DisasterList'):
            lDisasters = sd.sdGetVal('Disasters', 'DisasterList', 'Disasters')
            for pDisaster in lDisasters:
                if (iCounter % pDisaster.iPrimaryEffectTimer == 0 or iCounter == 0) and pDisaster.sPrimaryEffect != "NONE":
                    CyEngine().triggerEffect(gc.getInfoTypeForString(pDisaster.sPrimaryEffect), gc.getMap().plot(pDisaster.X,pDisaster.Y).getPoint())    
                if (iCounter % pDisaster.iSecondaryEffectTimer == 0 or iCounter == 0) and pDisaster.sSecondaryEffect != "NONE":
                    CyEngine().triggerEffect(gc.getInfoTypeForString(pDisaster.sSecondaryEffect), gc.getMap().plot(pDisaster.X, pDisaster.Y).getPoint())
                
                
    def checkSpawning(self):
        pTile = gc.getMap().plot(CyGame().getSorenRandNum(gc.getMap().getGridWidth(), ""),CyGame().getSorenRandNum(gc.getMap().getGridHeight(), ""))
        iDisasterYes = CyGame().getSorenRandNum(100, "chance to disaster")
        if iDisasterYes <= self.iBaseDisasterChance:
            lDisasterBoneses = sd.sdGetVal('Disasters','DisasterBonesList', 'DisasterBoneses')
            lWeights = []
            for pDisasterBones in lDisasterBoneses:
                iWeight = pDisasterBones.iWeight
                lWeights.append(iWeight)
            iTotalWeight = 0
            for i in range(len(lWeights)):
                iTotalWeight += lWeights[i]
            iWhich = CyGame().getSorenRandNum(iTotalWeight, "")
            iSoFarSum = 0
            for i in range(len(lWeights)):
                iSoFarSum += lWeights[i]
                if iWhich <= iSoFarSum:
                    self.spawnDisasterByBones(lDisasterBoneses[i], pTile.getX(), pTile.getY())
                
        
        
    def updateDisasters(self):
        if sd.sdEntityExists('Disasters', 'DisasterList'):
            lDisasters = sd.sdGetVal('Disasters', 'DisasterList', 'Disasters')
            for pDisaster in lDisasters:
                self.doDamage(pDisaster)
                self.doSpreading(pDisaster, lDisasters)
                self.doMoving(pDisaster)
                self.doExtinguishing(pDisaster, lDisasters)
            sd.sdSetVal('Disasters', 'DisasterList', 'Disasters', lDisasters)
        
    def doDamage(self, pDisaster):
        if len(pDisaster.lDamages) == 0:
            return
        for pDamage in pDisaster.lDamages:
            if not CyGame().getSorenRandNum(100, "") < pDamage.iChance:
                continue

            iCenterX = pDisaster.X
            iCenterY = pDisaster.Y
            if pDamage.iDamageCenter != 0:
                iCenterX = self.getRandomNumber(pDisaster.X - pDamage.iDamageCenter, pDisaster.X + pDamage.iDamageCenter)
                iCenterY = self.getRandomNumber(pDisaster.Y - pDamage.iDamageCenter, pDisaster.Y + pDamage.iDamageCenter)
            pCenter = gc.getMap().plot(iCenterX, iCenterY)
            for x in range(iCenterX - pDamage.iDamageMaxDistance, iCenterX + pDamage.iDamageMaxDistance + 1):
                for y in range(iCenterY - pDamage.iDamageMaxDistance, iCenterY + pDamage.iDamageMaxDistance + 1):
                    if not gc.getMap().isPlot(x,y):
                        continue
                    pTile = gc.getMap().plot(x,y)
                    if pDamage.sEffect != "NONE":
                        CyEngine().triggerEffect(gc.getInfoTypeForString(pDamage.sEffect), gc.getMap().plot(x,y).getPoint())    
                    if pTile.isCity():
                        pCity = pTile.getPlotCity()
                        for building in range(gc.getNumBuildingInfos()):
                            if pCity.isHasBuilding(building):
                                iActualBDamage = self.getActualDamage(pTile, pDamage, pCenter, pDamage.iBuildingMaxDamage, pDamage.iBuildingMinDamage)
                                if iActualBDamage > 0:
                                    self.doBuildingDamage(iActualBDamage, building, pCity)
                                    messagestring = pDamage.sDamageDescription
                                    messagestring += " did %d damage to the %s in %s!" % (iActualBDamage, gc.getBuildingInfo(building).getDescription(), pCity.getName())
                                    CyInterface().addMessage(pCity.getOwner(), False, 10, messagestring, "", -1, gc.getBuildingInfo(building).getButton(), 7, pCenter.getX(), pCenter.getY(), True, True)
                        iActualPDamage = self.getActualDamage(pTile, pDamage, pCenter, pDamage.iPopulationMaxDamage, pDamage.iPopulationMinDamage)
                        self.doPopulationDamage(iActualPDamage, pCity, pDamage)
                    iActualIDamage = self.getActualDamage(pTile, pDamage, pCenter, pDamage.iImprovementMaxDamage, pDamage.iImprovementMinDamage)
                    self.doImprovementDamage(iActualIDamage, pTile)
                    for i in range(pTile.getNumUnits()):
                        pUnit = pTile.getUnit(i)
                        iActualUDamage = self.getActualDamage(pTile, pDamage, pCenter, pDamage.iUnitMaxDamage, pDamage.iUnitMinDamage)
                        pUnit.changeDamage(iActualUDamage, pUnit.getOwner())
                        messagestring = pDamage.sDamageDescription
                        messagestring += " did %d damage to your %s!" % (iActualUDamage, pUnit.getName())
                        if iActualUDamage > 0:
                            CyInterface().addMessage(pUnit.getOwner(), False, 10, messagestring, "", -1, gc.getUnitInfo(pUnit.getUnitType()).getButton(), 7, pCenter.getX(), pCenter.getY(), True, True)

    def doMoving(self, pDisaster):
        if not pDisaster.iMoveSpeed > 0:
            return
        else:
            if pDisaster.iTargetX == -1:
                pTargetTile = self.findNewTarget(pDisaster)
                pDisaster.iTargetX = pTargetTile.getX()
                pDisaster.iTargetY = pTargetTile.getY()
            if pDisaster.X > pDisaster.iTargetX:
                pDisaster.X -= pDisaster.iMoveSpeed
            elif pDisaster.X < pDisaster.iTargetX:
                pDisaster.X += pDisaster.iMoveSpeed
            if pDisaster.Y > pDisaster.iTargetY:
                pDisaster.Y -= pDisaster.iMoveSpeed
            elif pDisaster.Y < pDisaster.iTargetY:
                pDisaster.Y += pDisaster.iMoveSpeed
            if pDisaster.X == pDisaster.iTargetX and pDisaster.Y == pDisaster.iTargetY:
                pDisaster.iTargetX = -1
        
    
    def findNewTarget(self, pDisaster):
        lPotentialTargets = []
        for x in range(pDisaster.X - self.iMaxSearchRange, pDisaster.X + self.iMaxSearchRange):
            for y in range(pDisaster.Y - self.iMaxSearchRange, pDisaster.Y + self.iMaxSearchRange):
                if not gc.getMap().isPlot(x,y):
                    continue
                pTile = gc.getMap().plot(x,y)
                if pDisaster.sTarget == "NONE":
                    ## tile will be chosen randomly unless the land thing matters
                    if pDisaster.bTargetLand:
                        if not pTile.isWater():
                            lPotentialTargets.append(pTile)
                    else:
                        lPotentialTargets.append(pTile)
                elif self.isOnTile(pTile, pDisaster.sTarget):
                    lPotentialTargets.append(pTile)
        CyInterface().addImmediateMessage("%s no. of potential targets %d" % (pDisaster.sDisasterTag, len(lPotentialTargets)),"")
        if not len(lPotentialTargets) > 0:
            ## Then the disaster's own tile is the target
            return gc.getMap().plot(pDisaster.X, pDisaster.Y)
        else:
            ind = CyGame().getSorenRandNum(len(lPotentialTargets),"")
            return lPotentialTargets[ind]
    
    def doSpreading(self, pDisaster, lDisasters):
        if not len(pDisaster.lSpreads) > 0:
            return
        for pSpread in pDisaster.lSpreads:
            if pSpread.bSpreadAdjacent:
                for x in range(pDisaster.X - 1, pDisaster.X + 2):
                    for y in range(pDisaster.Y - 1, pDisaster.Y + 2):
                        if x == pDisaster.X and y == pDisaster.Y:
                            continue
                        if not gc.getMap().plot(x,y):
                            continue
                        pTile = gc.getMap().plot(x,y)
                        iChance = self.getTotalSpreadChance(pSpread, pDisaster)
                        if CyGame().getSorenRandNum(100,"") <= iChance:
                            self.spawnDisasterByTypeList(pSpread.sSpreadDisasterType, x, y, lDisasters)
            if pSpread.bSpreadOwnTile:
                iChance = self.getTotalSpreadChance(pSpread, pDisaster)
                if CyGame().getSorenRandNum(100,"") <= iChance:
                    self.spawnDisasterByTypeList(pSpread.sSpreadDisasterType, pDisaster.X, pDisaster.Y, lDisasters)
            if pSpread.bTradeRouteSpread:
                if not gc.getMap().plot(pDisaster.X, pDisaster.Y).isCity():
                    return
                pCity = gc.getMap().plot(pDisaster.X, pDisaster.Y).getPlotCity()
                iTradeRoutes = pCity.getTradeRoutes()
                for i in range(iTradeRoutes):
                    tradeCity = pCity.getTradeCity(i)
                    if CyGame().getSorenRandNum(100, "") <= self.getTotalSpreadChance(pSpread, pDisaster):
                        self.spawnDisasterByTypeList(pSpread.sSpreadDisasterType, tradeCity.getX(), tradeCity.getY(), lDisasters)
                return

    def getTotalSpreadChance(self, pSpread, pDisaster):
        iChance = pSpread.iSpreadChance
        for increaser in pSpread.lSpreadChanceIncreasers:
            if self.isOnTile(gc.getMap().plot(pDisaster.X, pDisaster.Y), increaser):
                iChance += pSpread.lSpreadChanceIncreases[pSpread.lSpreadChanceIncreasers.index(increaser)]
        return iChance
                
        ## trade route spreading
        
    
    def doExtinguishing(self, pDisaster, lDisasters):
        pDisaster.iAge += 1
        if pDisaster.iAge >= pDisaster.iLifeTime:
            ## Time to delete the disaster
            lDisasters.pop(lDisasters.index(pDisaster))
            del pDisaster ## not sure if this line is necessary but I feel like it is
        
    
    def getActualDamage(self, pTile, pDamage, pCenter, iMax, iMin):
        iMaxDamage = self.getDamage(self.getDistance(pTile, pCenter), iMax, pDamage.bInverseSquareDropoff, pDamage.iLinearDropoffPercent)
        iMinDamage = self.getDamage(self.getDistance(pTile, pCenter), iMin, pDamage.bInverseSquareDropoff, pDamage.iLinearDropoffPercent)
        iActualDamage = self.getRandomNumber(iMinDamage, iMaxDamage)
        return iActualDamage


    def doBuildingDamage(self, iDamage, iBuilding, pCity):
        ## First remove the building
        pCity.setNumRealBuilding(iBuilding, 0)
        ## Now put production for the building back
        fRealDamage = float(iDamage) / 100.0
        iTotalHammers = gc.getBuildingInfo(iBuilding).getProductionCost()
        iRealHammers = iTotalHammers - int(fRealDamage * float(iTotalHammers))
        pCity.setBuildingProduction(iBuilding, iRealHammers)
            
    def doPopulationDamage(self, iDamage, pCity, pDamage):
        iPopulation = pCity.getPopulation()
        fDamage = float(iDamage) / 100.0
        iPopulationRemoved = int(fDamage * float(iPopulation))
        if iPopulationRemoved == iPopulation:
            iPopulationRemoved -= 1
        if iPopulationRemoved > 0: ## never know, could end up negative
            pCity.changePopulation(-1*iPopulationRemoved)
            messagestring = pDamage.sDamageDescription
            messagestring += " killed %d population in %s!" % (iPopulationRemoved, pCity.getName())
            CyInterface().addMessage(pCity.getOwner(), False, 10, messagestring, "", -1, gc.getBuildingInfo(building).getButton(), 7, pCity.getX(), pCity.getY(), True, True)

    
    def doImprovementDamage(self, iDamage, pTile):
        ## I may introduce 'damaged improvements' later but for now this is just a chance to pillage the improvement
        improvement = pTile.getImprovementType()
        if improvement != -1:
            if CyGame().getSorenRandNum(100, "") < iDamage:
                pTile.setImprovementType(-1)
    
    def isOnTile(self, pTile, sThingString):
        if "IMPROVEMENT_" in sThingString:
            if pTile.getImprovementType() == gc.getInfoTypeForString(sThingString):
                return True
            else:
                return False
        elif "FEATURE_" in sThingString:
            if pTile.getFeatureType() == gc.getInfoTypeForString(sThingString):
                return True
            else:
                return False
        elif "TERRAIN_" in sThingString:
            if pTile.getTerrainType() == gc.getInfoTypeForString(sThingString):
                return True
            else:
                return False
        elif "BONUS_" in sThingString:
            if pTile.getBonusType() == gc.getInfoTypeForString(sThingString):
                return True
            else:
                return False
        elif "BUILDING_" in sThingString:
            if not pTile.isCity():
                return False
            pCity = pTile.getPlotCity()
            for building in range(gc.getNumBuildingInfos()):
                if pCity.isHasBuilding(building) and gc.getBuildingInfo(building).getBuildingType() == gc.getInfoTypeForString(sThingString):
                    return True
            return False
        elif "UNIT_" in sThingString:
            if pTile.getNumUnits() > 0:
                for iUnit in range(pTile.getNumUnits()):
                    pUnit = pTile.getUnit(iUnit)
                    if pUnit.getUnitType == gc.getInfoTypeForString(sThingString):
                        return True
                return False
        else:
            CyInterface().addImmediateMessage("Debug: isOnTile can't read type", "")
    
    
    def getRandomNumber(self, iMin, iMax, bVary = False):
        iRandNum = CyGame().getSorenRandNum(iMax - iMin + 1, "random") + iMin
        if bVary:
            iRandNum = self.getRandomNumber(iRandNum - self.iDamageVariance, iRandNum + self.iDamageVariance, False)
        return iRandNum
    
    def getDamage(self, iDistance, iDamage, bInverse = True, iLinearDrop = 0):
        fDamage = float(iDamage)
        if bInverse and iDistance != 0:
            fDamage /= float(iDistance ** 2)
        elif iLinearDrop > 0:
            fDamage -= (iLinearDrop * iDistance)
        else:
            fDamage = iDamage
        return int(fDamage)
    
    def getDistance(self, pTile1, pTile2):
        x1 = pTile1.getX()
        x2 = pTile2.getX()
        y1 = pTile1.getY()
        y2 = pTile2.getY()
        dist = math.sqrt((x1-x2)**2+(y1-y2)**2)
        return int(dist)
    
    ## This next method was pilfered from The_J's SpawnACiv python mod. 
    ## I used that mod to learn how to load XML and this was a very nifty method to help with that. 
    ## It just cuts out the value of a line in XML and returns it. 
    def CutString(self, string):
        string = str(string)        
        string = string.strip()        
        string = string[2:-1]
        BeginPos=-1
        EndPos = -1                
        for i in xrange(len(string)):
            if string[i]==">":
                BeginPos=i                                
            elif string[i]=="<":
                EndPos=i                                
                break
        else:
            return "-1"
        NewString = string[BeginPos+1:EndPos]        
        return str(NewString)    
    
    
class DisasterBones(object):
    def __init__(self):
        ## This class is loaded as the XML setup for any particular disaster. 
        self.sDisasterTag = ""
        self.sDisasterDescription = ""
        self.iWeight = 0
        self.sPrimaryEffect = ""
        self.iPrimaryEffectTimer = -1
        self.sSecondaryEffect = ""
        self.iSecondaryEffectTimer = -1
        self.lDamages = []
        self.lSpreads = []
        self.iMinLifeTime = 0
        self.iMaxLifeTime = 0
        self.iMoveSpeed = 0
        self.sTarget = "NONE"
        self.bTargetLand = False
    
class Disaster(object):
    def __init__(self, pDisasterBones):
        self.sDisasterTag = pDisasterBones.sDisasterTag
        self.sDisasterDescription = pDisasterBones.sDisasterDescription
        self.X = -1
        self.Y = -1
        self.iMinLifeTime = pDisasterBones.iMinLifeTime
        self.iMaxLifeTime = pDisasterBones.iMaxLifeTime
        self.iAge = 0
        iLifeSpread = self.iMaxLifeTime - self.iMinLifeTime
        self.iLifeTime = CyGame().getSorenRandNum(iLifeSpread + 1, "") + self.iMinLifeTime
        self.sPrimaryEffect = pDisasterBones.sPrimaryEffect
        self.iPrimaryEffectTimer = pDisasterBones.iPrimaryEffectTimer
        self.sSecondaryEffect = pDisasterBones.sSecondaryEffect
        self.iSecondaryEffectTimer = pDisasterBones.iSecondaryEffectTimer
        self.lDamages = pDisasterBones.lDamages
        self.lSpreads = pDisasterBones.lSpreads
        self.iMoveSpeed = pDisasterBones.iMoveSpeed
        self.sTarget = pDisasterBones.sTarget
        self.bTargetLand = pDisasterBones.bTargetLand
        self.iTargetX = -1
        self.iTargetY = -1
        
        
class Damage(object):
    ## This class is a single damage effect. Disasters can have as many as they need
    def __init__(self):
        self.sDamageDescription = ""
        self.iBuildingMinDamage = 0
        self.iBuildingMaxDamage = 0
        self.iUnitMinDamage = 0
        self.iUnitMaxDamage = 0
        self.iImprovementMaxDamage = 0
        self.iImprovementMinDamage = 0
        self.iPopulationMaxDamage = 0
        self.iPopulationMinDamage = 0
        self.iDamageMaxDistance = 0
        self.iDamageCenter = 0
        self.bInverseSquareDropoff = False
        self.iLinearDropoffPercent = 0
        self.iChance = 0
        self.sEffect = "NONE"
        
class Spread(object):
    ## Single spread effect
    def __init__(self):
        self.sSpreadDisasterType = ""
        self.bTradeRouteSpread = False
        self.bSpreadOwnTile = False
        self.bSpreadAdjacent = False
        self.iSpreadChance = 0
        self.iSpreadIncreasePerPop = 0
        self.lSpreadChanceIncreasers = []
        self.lSpreadChanceIncreases = []